/**
 * Application settings content script.
 *
 * @module content-scripts/settings
 * @listens worker:initialized
 * @listens setting:fetched
 */

'use strict';

/**
 * Fires once the content script worker is initialized.
 *
 * @param {Object} initialState The initial state of the settings module.
 */
self.port.once('worker:initialized', function (initialState) {

    applyL10n(initialState.l10n);
    applySettings(initialState.settings);
});

/**
 * Fires once a setting has been fetched.
 *
 * @param {Object} setting The fetched application setting.
 */
self.port.on('setting:fetched', function (setting) {
    applySetting(setting);
});

/**
 * Applies a given localization to marked document elements.
 * https://bugzilla.mozilla.org/show_bug.cgi?id=787351
 *
 * @param {Object} l10n A localization object.
 * @private
 */
function applyL10n (l10n) {

    let l10nKeys = Object.keys(l10n);

    l10nKeys.forEach(function (l10nKey) {

        let elements = document.querySelectorAll('[data-l10n-id="' + l10nKey + '"]');

        for (let element of elements) {
            element.textContent = l10n[l10nKey];
        }
    });
}

/**
 * Applies a collection of settings to the user interface.
 *
 * @param {Object} settings A collection of application setting objects.
 * @private
 */
function applySettings (settings) {

    let settingIds = Object.keys(settings);

    settingIds.forEach(function (settingId) {

        let setting = settings[settingId];
        applySetting(setting);
    });
}

/**
 * Applies a given setting to the user interface.
 *
 * @param {Setting} setting An application setting object.
 * @private
 */
function applySetting (setting) {

    let elements = document.querySelectorAll('[data-setting-id="' + setting.id + '"]');

    for (let element of elements) {

        element.setAttribute('readonly', 'readonly');
        element.value = setting.value;
        element.removeAttribute('readonly');
    }
}

/**
 * Fetch setting event emitter.
 *
 * @event setting:fetch
 * @property {String} settingId The identifier of the requested setting.
 */
function fetchSetting (settingId) {
    self.port.emit('setting:fetch', settingId);
}

/**
 * Setting altered event emitter.
 *
 * @event setting:altered
 * @property {SettingAlteration} settingAlteration The setting alteration details.
 */
function settingAltered (settingAlteration) {
    self.port.emit('setting:altered', settingAlteration);
}
